@@ -19,6 +19,20 @@ from requests.auth import AuthBase
 
 from .cache import RequestCache
 
+TRUE_VALUES = (True, 'true', 't', 'yes', 'y')
+FALSE_VALUES = (False, 'false', 'f', 'no', 'n')
+
+
+def convert_bool(value):
+    if isinstance(value, str):
+        value = value.lower()
+    if value in TRUE_VALUES:
+        return True
+    elif value in FALSE_VALUES:
+        return False
+    else:
+        raise ValueError('Wrong boolean value')
+
 
 class AWSSigV4(AuthBase):
 
@@ -208,13 +222,16 @@ class BaseClient:
             return demjson.decode(response.text)
 
     def request(self, path: str, *, data: Union[dict, bytes] = None, params: dict = None, headers=None,
-                method='GET') -> Response:
+                method='GET', check_exception=True) -> Response:
 
+        # process params
         parsed_params = {}
-        parsed_data: bytes = b''
         params is None or parsed_params.update(params)
+
+        # process data
+        parsed_data: bytes
         if data is None:
-            parsed_data = json.dumps({}).encode()
+            parsed_data = b''
         elif isinstance(data, dict):
             parsed_data = json.dumps(data).encode()
         elif isinstance(data, bytes):
@@ -222,12 +239,14 @@ class BaseClient:
         else:
             raise TypeError('data should be a dict or bytes')
 
+        # process cache, will be removed in next main version
         if self.use_cache:
             cache_keys: RequestCache.KEYS_TYPE = (path, method, params, data, headers)
             response = self.request_cache[cache_keys]
             if response is not None:
                 return response
 
+        # process headers
         parsed_headers = {
             'host': self._endpoint[8:],
             'user-agent': 'python-sp-api',
@@ -235,30 +254,28 @@ class BaseClient:
             'x-amz-date': datetime.utcnow().strftime('%Y%m%dT%H%M%SZ'),
             'content-type': 'application/json'
         }
-
         headers is None or parsed_headers.update(headers)
 
+        # process auth
         if 'role' not in self._role_cache:
             role = self._client.assume_role(RoleArn=self._role_arn, RoleSessionName='guid').get('Credentials')
             self._role_cache['role'] = role
         role = self._role_cache['role']
-
         auth = AWSSigV4('execute-api',
                         aws_access_key_id=role.get('AccessKeyId'),
                         aws_secret_access_key=role.get('SecretAccessKey'),
                         region=self._region,
                         aws_session_token=role.get('SessionToken'))
-        while True:
+        url = self._endpoint + path
 
-            if method == 'POST':
-                response = request(method, self._endpoint + path, data=parsed_data, headers=parsed_headers,
-                                   auth=auth)
-            elif method == 'GET':
-                response = request(method, self._endpoint + path, params=params, headers=parsed_headers, auth=auth)
-            else:
-                raise ValueError('unknown method')
+        # process quota exceeded exception
+        while True:
+            response = request(method=method, url=url, data=parsed_data, headers=parsed_headers,
+                               auth=auth, params=params)
 
             self.last_response = response
+            if not check_exception:
+                break
 
             # If found error, and the error is QuotaExceed, just resend the request
             e = self._get_response_json(response).get('errors', None)
@@ -271,8 +288,20 @@ class BaseClient:
             else:
                 break
 
+        # process cache, will be removed in the next main version
         if self.use_cache:
             cache_keys: RequestCache.KEYS_TYPE = (path, method, params, data, headers)
             self.request_cache[cache_keys] = response
 
         return response
+
+
+class BaseDictObject:
+    def __init__(self, data):
+        self.data: dict = data
+
+    def _get_value(self, klass: type, name: str):
+        value = self.data.get(name, None)
+        if value is None and klass is dict:
+            return None
+        return klass(value)
